package com.nisum.spring.kafka.api;

import java.time.Instant;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.CountDownLatch;

import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.event.EventListener;
import org.springframework.http.MediaType;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.Disposable;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.core.scheduler.Scheduler;
import reactor.core.scheduler.Schedulers;
import reactor.kafka.receiver.KafkaReceiver;
import reactor.kafka.receiver.ReceiverOffset;
import reactor.kafka.receiver.ReceiverOptions;
import reactor.kafka.receiver.ReceiverRecord;

@SpringBootApplication
@RestController
public class KafkaConsumerApplication {
    private static final Logger log = LoggerFactory.getLogger(KafkaConsumerApplication.class.getName());
    private String consumedData;
    List<String> messages = new ArrayList<>();

    User userFromTopic = null;

    private String topic = "venkatPersonalTopic";

    @Autowired
    ReceiverOptions receiverOptions;
    private final DateTimeFormatter dateFormat = DateTimeFormatter.ofPattern("HH:mm:ss:SSS z dd MMM yyyy");

    @GetMapping("/consumeStringMessage")
    public List<String> consumeMsg() {
        return messages;
    }

    @GetMapping("/consumeJsonMessage")
    public User consumeJsonMessage() {
        return userFromTopic;
    }

    @KafkaListener(groupId = "stringGroup", topics = "venkatPersonalTopic", containerFactory = "kafkaListenerContainerFactory")
    public List<String> getMsgFromTopic(String data) {
        messages.add(data);
        return messages;
    }

    @KafkaListener(groupId = "jsonGroup", topics = "venkatPersonalTopic", containerFactory = "userKafkaListenerContainerFactory")
    public User getJsonMsgFromTopic(User user) {
        userFromTopic = user;
        return userFromTopic;
    }

    //TEXT_EVENT_STREAM_VALUE
    @GetMapping(name = "/consumeReactorJsonMessage", produces = MediaType.APPLICATION_STREAM_JSON_VALUE)
    public String consumeMessages() {

        ReceiverOptions subscription = receiverOptions.subscription(Collections.singleton(topic));
        Flux<ReceiverRecord<Integer, String>> inboundFlux =
                KafkaReceiver.create(receiverOptions)
                        .receive();
        inboundFlux.subscribe(
                data -> setConsumedData(data.value()), // onNext
                err -> onError(err),  // onError
                () -> onComplete() // onComplete
        );
        return getConsumedData();
    }

    private static <T> void onError(Throwable err) {
        System.out.println("onError: Exception occurred: " + err.getMessage());
    }

    private static <T> void onComplete() {
        System.out.println("onComplete: Completed!");
    }
    public String getConsumedData() {
        return consumedData;
    }

    public void setConsumedData(String consumedData) {
        this.consumedData = consumedData;
    }

    public static void main(String[] args) {
        SpringApplication.run(KafkaConsumerApplication.class, args);
    }
}
