package com.nisum.reactive.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.nisum.reactive.dto.ProductDto;
import com.nisum.reactive.dto.User;
import com.nisum.reactive.service.ProductDBService;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.publisher.Mono;
import reactor.kafka.sender.KafkaSender;

@SpringBootApplication
@RestController
public class ProductKafkaController {
    private static final Logger log = LoggerFactory.getLogger(com.nisum.reactive.controller.ProductKafkaController.class.getName());
    @Autowired
    private KafkaTemplate<String, Object> template;
    @Autowired
    ProductDBService service;
    @Autowired
    private KafkaSender<String, Object> kafkaSender;
    @Autowired
    private ObjectMapper objectMapper;
    private String topic = "venkatPersonalTopic";

    @GetMapping("/publish/{name}")
    public String publishMessage(@PathVariable String name) {
        template.send(topic, "Hi " + name + " Welcome to java Nisum");
        return "Data published";
    }

    @GetMapping("/publishJson")
    public ProductDto publishMessage(@RequestBody ProductDto user) {
        template.send(topic, user);
        return user;
    }

    @GetMapping("/publishReactorJson")
    public Mono<User> sendMessages(@RequestBody User user) throws InterruptedException, JsonProcessingException {
        return this.kafkaSender.createOutbound()
                .send(Mono.just(new ProducerRecord<>(topic,
                        "dto.getSalt()", user))).then()
                .doOnError(e -> log.error(
                        String.format("Failed to send topic: %s value: %s",
                                topic, user), e))
                .thenReturn(user);
    }


}
