package com.nisum.reactive.routerHandler;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.nisum.reactive.KafkaPublisherApplication;
import com.nisum.reactive.dto.ProductDto;
import com.nisum.reactive.dto.User;
import com.nisum.reactive.service.ProductDBService;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.reactive.function.BodyExtractors;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.server.ServerRequest;
import org.springframework.web.reactive.function.server.ServerResponse;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.kafka.sender.KafkaSender;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Optional;

@Service
public class ProductKafkaHndler {
    private static final Logger log = LoggerFactory.getLogger(ProductKafkaHndler.class.getName());
    @Autowired
    private KafkaTemplate<String, Object> template;
    @Autowired
    ProductDBService service;
    @Autowired
    private KafkaSender<String, Object> kafkaSender;
    @Autowired
    private ObjectMapper objectMapper;
    private String topic = "venkatPersonalTopic";

    private static final String BASEURL = "http://localhost:9091/publishReactorJson";
    public Mono<ServerResponse> publishMessage(ServerRequest request) {
        String name = request.pathVariable("name");
        template.send(topic, "Hi " + name + " Welcome to java Nisum");
        return ServerResponse.ok().body( Mono.just("Data Published"), String.class);
    }

    @GetMapping("/publishJson")
    public ProductDto publishMessage(@RequestBody ProductDto user) {
        //User user = new User(2532, "User88", new String[] { "Bangalore", "BTM", "house 90" });
        template.send(topic, user);
        return user;
    }

    public Mono<ServerResponse> getAllProducts(ServerRequest request) {
        return ServerResponse.ok().body(service.getProducts(), ProductDto.class);
    }

    public Mono<ServerResponse> findProduct(ServerRequest request) {
        //User user = new User(2532, "User88", new String[]{"Bangalore", "BTM", "house 90"});
        String id = request.pathVariable("id");
        Mono<ProductDto> product = service.getProduct(id);
        return ServerResponse.ok().body(product, ProductDto.class);
    }

    public Mono<ServerResponse> saveProduct(ServerRequest request) {
       // Mono<ProductDto> productMono = request.bodyToMono(ProductDto.class);
        Mono<ProductDto> saveResponse = service.saveProduct(request.bodyToMono(ProductDto.class));
        return ServerResponse.ok().body(saveResponse, ProductDto.class);
    }

    public Mono<ServerResponse> updateProduct(ServerRequest request) {
        String id = request.pathVariable("id");
        Mono<ProductDto> productMono = request.bodyToMono(ProductDto.class);
        Mono<ProductDto> product = service.updateProduct(productMono, id);
        return ServerResponse.ok().body(product, ProductDto.class);
    }

    public Mono<ServerResponse> deleteProduct(ServerRequest request) {
        String id = request.pathVariable("id");
        Mono<Void> voidMono = service.deleteProduct(id);
        return ServerResponse.ok().body(voidMono, Void.class);
    }

}
