package com.nisum.reactive.service;

import com.nisum.reactive.dto.ProductDto;
import com.nisum.reactive.repository.ProductRepository;
import com.nisum.reactive.utils.AppUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Range;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class ProductDBService {

    @Autowired
    private ProductRepository repository;


    public Flux<ProductDto> getProducts(){
        return repository.findAll().map(AppUtils::entityToDto);
    }

    public Mono<ProductDto> getProduct(String id){
        return repository.findById(id).map(AppUtils::entityToDto);
    }

    public Flux<ProductDto> getProductInRange(double min,double max){
        final Range.Bound<Double> lowerBound = Range.Bound.inclusive(min);
        final Range.Bound<Double> upperBound = Range.Bound.inclusive(max);
        return repository.findByPriceBetween(Range.of(lowerBound,upperBound));
    }

    public Mono<ProductDto> saveProduct(Mono<ProductDto> dto){
        //Mono<ProductDto> productDtoMono = Mono.justOrEmpty(dto);
        System.out.println("service method called ...");
      return  dto.map(AppUtils::dtoToEntity)
                .flatMap(repository::insert)
                .map(AppUtils::entityToDto);
    }

    public Mono<ProductDto> updateProduct(Mono<ProductDto> dto,String id){
        //Mono<ProductDto> productDtoMono = Mono.justOrEmpty(dto);
        return repository.findById(id)
                .flatMap(p->dto.map(AppUtils::dtoToEntity)
                .doOnNext(e->e.setId(id)))
                .flatMap(repository::save)
                .map(AppUtils::entityToDto);

    }

    public Mono<Void> deleteProduct(String id){
        return repository.deleteById(id);
    }
}
